/*******************************************************************
 Copyright (C) 2011-2024  Patrick H. E. Foubet - S.E.R.I.A.N.E.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
*******************************************************************/

/* fonctions d'execution de commandes externes */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <string.h>
#include <errno.h>

#include "trace.h"
#include "comsh.h"
#include "lich.h"
#include "debug.h"

/* variables globales maj dans main() */
pid_t pidG;
char * NPROG;

void prAube(void)
{
static char *c;
static int i=0;
   c=Y(f03);
   printf(Y(f04),c[i++]);
   fflush(stdout);
   if (i==4) i=0;
}

void Pignore(char * m) { printf(Y(a01),m); }

#define LRES 100
static char * comshr(char *b)
{
pid_t pid;
int n, ret, pic[2];
static char res[LRES];
char *d,*f;
   if (pipe(pic) < 0) {
      perror(Y(p01)); return NULL;
   }
   fcntl(pic[0], F_SETFL, O_NONBLOCK);
   if ((pid = fork()) < 0) {
      perror(Y(p02)); return NULL;
   }
   if (pid == 0) {
      close(pic[0]);
      fcntl(pic[1], F_SETFL, O_NONBLOCK);
      dup2(pic[1],1);
      signal(SIGINT,SIG_IGN);
      execl(Y(c02), Y(k00), "-c", b, (char*)0);
      perror(Y(p03));
      printf(Y(k10));
      exit(0);
   }
   close(pic[1]);
   d=res;
   f=d+LRES;
   while (d < f) {
       if ((n=read(pic[0],d,1)) < 1) {
          if (errno == EAGAIN) continue;
          else break;
       }
       if (*d == '\n') break;
       d++;
   }
   if (d==f) {
      d--;
      while (*d != '\n') if (read(pic[0],d,1) != 1) break;
   }
   *d = '\0';
   kill(pid,SIGKILL);
   waitpid(pid,&ret,0);
   close(pic[0]);
   return res;
}

/* fcts utilisant comshr */
char * comWhois(char * ip)
{
char a[50], b[150],*d;
   strcpy(a,ip);
   d=strrchr(a,(int)'/');
   if (d != NULL) *d='\0';
   sprintf(b,Y(f38), Y(c03), a, Y(c04));
   return comshr(b);
}

int comsh(char *com,int mode)
{
pid_t pid;
int ret;
   Tdebug(Y(t09),com);
   if ((pid = fork()) < 0) {
      perror(Y(p04)); return 99;
   }
   if (pid == 0) {
      if (mode & EX_NOOUT) close(1);
      if (mode & EX_NOERR) close(2);
      signal(SIGINT,SIG_IGN);
      execl(Y(c02), Y(k00), "-c", com, (char *)0);
      perror(Y(p05)); exit(98);
   }
   if (mode & EX_DUREE) {
      while (waitpid(pid,&ret,WNOHANG) != pid) {
        sleep(1);
        prAube();
      }
   } else waitpid(pid,&ret,0);
   return WEXITSTATUS(ret);
}

int exeComA(char * comm)
{
char b[120];
   sprintf(b,Y(f39),comm);
   return comsh(b,0);
}

/* ### fct de MAJ iptables */

void razIPT(void)
{
char b[90];
   /* IPv4 */
   sprintf(b,Y(f40),Y(c05));
   exeComA(b);
   /* IPv6 */
   sprintf(b,Y(f40),Y(c06));
   exeComA(b);
}

int initIPT(void)
{
int i=0;
char b[90];
   razIPT();
   /* IPv4 */
   sprintf(b,Y(f41),Y(c05),Y(k01));
   if (exeComA(b)) {
      sprintf(b,Y(f42),Y(c05),Y(k01));
      i += exeComA(b);
   }
   sprintf(b,Y(f43),Y(c05),Y(k03),Y(k01));
   i += exeComA(b);
   /* IPv6 */
   sprintf(b,Y(f41),Y(c06),Y(k02));
   if (exeComA(b)) {
      sprintf(b,Y(f42),Y(c06),Y(k02));
      i += exeComA(b);
   }
   sprintf(b,Y(f43),Y(c06),Y(k03),Y(k02));
   i += exeComA(b);
   return i;
}

static int isPresentIP(char * comm, char * ip, char * chain)
{
char buf[100];
   sprintf(buf,Y(f44),comm,chain,ip);
   if (exeComA(buf) == 0) return 1;
   return 0;
}

/* fct derivees de isPresentIP */
int isIPprotegee(int v6, char *ip)
{
   if (v6) return isPresentIP(Y(c06),ip,Y(k02));
   else    return isPresentIP(Y(c05),ip,Y(k01));
}
int isIPbloquee(int v6, char *ip)
{
   if (v6) return isPresentIP(Y(c06),ip,Y(k03));
   else    return isPresentIP(Y(c05),ip,Y(k03));
}


static int retireChain(char * comm, char * ip, char * chain, char * jump)
{
char buf[100];
   sprintf(buf,Y(f45),comm,chain, ip, jump);
   return exeComA(buf);
}
/* fct derivees de retireChain() */
int derejetIP(int v6, char *ip)
{
   if (v6) return retireChain(Y(c06),ip,Y(k03),Y(k04));
   else    return retireChain(Y(c05),ip,Y(k03),Y(k04));
}
int deprotegeIP(int v6, char *ip)
{
   if (v6) return retireChain(Y(c06),ip,Y(k02),Y(k05));
   else    return retireChain(Y(c05),ip,Y(k01),Y(k05));
}

static int ajouteChain(char * comm, char * ip, char * chain, char * jump)
{
char buf[100];
   sprintf(buf,Y(f46),comm,chain, ip, jump);
   return exeComA(buf);
}
/* fct derivees de ajouteChain() */
int rejetIP(int v6, char *ip)
{
   if (v6) return ajouteChain(Y(c06),ip,Y(k03),Y(k04));
   else    return ajouteChain(Y(c05),ip,Y(k03),Y(k04));
}
int protegeIP(int v6, char *ip)
{
   if (v6) return ajouteChain(Y(c06),ip,Y(k02),Y(k05));
   else    return ajouteChain(Y(c05),ip,Y(k01),Y(k05));
}

/* commandes interactives */


void comFct(int C, int JCTL, char *cmd)
{
int code=0;
char com[200];
  switch(C) {
    case FCT_COMe:
      if (JCTL) sprintf(com,Y(f05),Y(c01),NPROG,pidG, Y(k08), Y(c14),Y(c07));
      else sprintf(com,Y(f06),NPROG,pidG, Y(k08), Y(k09),Y(c14),Y(c07));
      break;
    case FCT_COME:
      if (JCTL) sprintf(com,Y(f07),Y(c01),NPROG,pidG, Y(k11), Y(c15));
      else sprintf(com,Y(f08),NPROG,pidG, Y(k11), Y(k09),Y(c15));
      break;
    case FCT_COML:
      if (*(cmd+1) == '\0') {
         if (JCTL) sprintf(com,Y(f09),Y(c01),NPROG, pidG,Y(f13),Y(c15));
         else sprintf(com,Y(f10),NPROG,pidG, Y(c13),Y(f13),Y(c15));
      } else {
         if (JCTL) sprintf(com,Y(f11),Y(c01) ,NPROG,pidG,Y(f13),Y(c15),cmd+1);
         else sprintf(com,Y(f12),NPROG,pidG, Y(c13),Y(f13),Y(c15),cmd+1);
      }
      break;
    case FCT_COMl:
      if (*(cmd+1) != '\0') { /* avec parametre */
         if ((*(cmd+1) == '=') && (*(cmd+2) == '\0')) {
            printf(Y(e06));
            return;
         }
         if ((*(cmd+1) == '=') && (*(cmd+2) != '\0')) { /* script + param */
            sprintf(com,Y(f14),NPROG,pidG,cmd+2,pidG);
            comsh(com,0);
            sprintf(com,Y(f15),pidG,pidG);
         } else {
            if (JCTL)
            sprintf(com,Y(f16),Y(c01),NPROG,pidG, cmd+1,Y(f32),Y(c16));
            else sprintf(com,Y(f17),NPROG,pidG,cmd+1, Y(c13),Y(f32),Y(c16));
         }
      } else {
         if (JCTL)sprintf(com,Y(f18),Y(c01),NPROG,pidG, Y(f32),Y(c16));
         else sprintf(com,Y(f19),NPROG,pidG, Y(c13),Y(f32),Y(c16));
      }
      break;
    case FCT_COMinf:
      if (JCTL) sprintf(com,Y(f20),Y(c01),NPROG,pidG, Y(f33),Y(c17),Y(c09));
      else sprintf(com,Y(f21),NPROG,pidG, Y(c13),Y(f34),Y(c17),Y(c09));
      break;
    case FCT_COMsup:
      if (JCTL) sprintf(com,Y(f22),Y(c01),NPROG,pidG, Y(f35),Y(c17),Y(c09));
      else sprintf(com,Y(f23),NPROG,pidG, Y(c13),Y(f36),Y(c17),Y(c09));
      break;
    case FCT_COMr:
      sprintf(com,Y(f24),Y(c05),Y(k03),Y(k06),Y(c18),Y(c10),Y(c11));
      if (*(cmd+1) != '\0') {
         if ((*(cmd+1) != '?') || (*(cmd+2) != '\0')) Pignore(cmd+1);
         else sprintf(com,Y(f25),Y(c05),Y(k03),Y(k06),Y(c18),Y(c07));
      }
      code = EX_NOERR|EX_DUREE;
      break;
    case FCT_COMR:
      sprintf(com,Y(f26),Y(c06),Y(k06),Y(c19),Y(c08),Y(c12));
      if (*(cmd+1) != '\0') {
         if ((*(cmd+1) != '?') || (*(cmd+2) != '\0')) Pignore(cmd+1);
         else sprintf(com,Y(f27),Y(c06),Y(k06),Y(c19),Y(c07));
      }
      code = EX_NOERR|EX_DUREE;
      break;
    case FCT_COMv:
      sprintf(com,Y(f28),Y(c05),Y(k01),Y(k07),Y(c18),Y(c10),Y(c11));
      if (*(cmd+1) != '\0') {
         if ((*(cmd+1) != '?') || (*(cmd+2) != '\0')) Pignore(cmd+1);
         else sprintf(com,Y(f29),Y(c05),Y(k01),Y(k07),Y(c18),Y(c07));
      }
      code = EX_NOERR|EX_DUREE;
      break;
    case FCT_COMV:
      sprintf(com,Y(f30),Y(c06),Y(k02),Y(k07),Y(c19),Y(c08),Y(c12));
      if (*(cmd+1) != '\0') {
         if ((*(cmd+1) != '?') || (*(cmd+2) != '\0')) Pignore(cmd+1);
         else sprintf(com,Y(f31),Y(c06),Y(k02),Y(k07),Y(c19),Y(c07));
      }
      code = EX_NOERR|EX_DUREE;
      break;
    default:
      printf(Y(a02),C);
      return;
  }
  if (DebugOn) printf(Y(f37),com);
  comsh(com,code);
}

