/*******************************************************************
 Copyright (C) 2011-2024  Patrick H. E. Foubet - S.E.R.I.A.N.E.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
*******************************************************************/

/* gesip.c : fonctions pour la gestion des adresses IPv4 et IPv6 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>

#include "gesip.h"
#include "lich.h"

static int isIP(char *a)
{
struct in_addr S;
struct in6_addr S6;
int r;
   if ((r = inet_pton(AF_INET,a, (void *)&S)) == 1) return IP4;
   if ((r = inet_pton(AF_INET6,a, (void *)&S6)) == 1) return IP6;
   return ERR;
}

static int isIPv4(char *a)
{
struct in_addr S;
int r;
   if ((r = inet_pton(AF_INET,a, (void *)&S)) <= 0) return 0;
   return 1;
}

static int isIPv6(char *a)
{
struct in6_addr S;
int r;
   if ((r = inet_pton(AF_INET6,a, (void *)&S)) <= 0) return 0;
   return 1;
}

static void And6(unsigned char *r, unsigned char *a, unsigned char *b) { /* r=a&b */
int i;
    for(i = 0; i < 16; i++) r[i] = a[i] & b[i];
}

int valCidr(char * c)
{
char * sn;
   if ((sn=strchr(c,(int)'/')) == NULL) return 0;
   return atoi(sn+1);
}

int isCidr(char*r)
{
char buf[20], *sn;
int n;
   if (strlen(r)>18) return 0;
   strcpy(buf,r);
   if ((sn=strchr(buf,(int)'/')) == NULL) return 0;
   *sn = '\0';
   if (!isIPv4(buf)) return 0;
   n = atoi(sn+1);
   if (n>32) return 0;
   return n;
}

int isCidr6(char*r)
{
char buf[52], *sn;
int n;
   if (strlen(r)>50) return 0;
   strcpy(buf,r);
   if ((sn=strchr(buf,(int)'/')) == NULL) return 0;
   *sn = '\0';
   if (!isIPv6(buf)) return 0;
   n = atoi(sn+1);
   if (n>128) return 0;
   return n;
}

int isSousRes(char*r, char *a)
{
struct in_addr Sa, Sr, Sm;
struct in6_addr S6a, S6r, S6m, S6am, S6rm;
int n, m, i, imax, v6;
unsigned char M[16];
char buf[52],smasq[42],*sn;
   if ((v6=isIP(a)) == ERR) return 0;
   imax = 4 + (12*v6);
   strcpy(buf,r);
   if ((sn=strchr(buf,(int)'/')) == NULL) return 0;
   *sn = '\0';
   n = atoi(sn+1);
   /* calcul du masq */
   m=n;
   for (i=0;i<imax;i++) {
      if (m>7) { M[i]=255; m-=8;
      } else { M[i]=0;
         while (m>=0) M[i] |= 0x80 >> --m;
      }
   }
   if (v6) {
      sprintf(smasq,Y(f01), M[0],M[1],M[2],M[3],M[4],M[5],M[6],M[7],
             M[8],M[9],M[10],M[11],M[12],M[13],M[14],M[15]);
      inet_pton(AF_INET6,smasq, (void *)&S6m);
      inet_pton(AF_INET6,a, (void *)&S6a);
      inet_pton(AF_INET6,buf, (void *)&S6r);
      And6(S6rm.s6_addr, S6r.s6_addr, S6m.s6_addr);
      And6(S6am.s6_addr, S6a.s6_addr, S6m.s6_addr);
      if (bcmp(S6rm.s6_addr,S6am.s6_addr,16)==0) return 1;
   } else {
      sprintf(smasq,Y(f02),M[0],M[1],M[2],M[3]);
      inet_pton(AF_INET,smasq, (void *)&Sm);
      inet_pton(AF_INET,a, (void *)&Sa);
      inet_pton(AF_INET,buf, (void *)&Sr);
      if ((Sr.s_addr & Sm.s_addr) == (Sa.s_addr & Sm.s_addr)) return 1;
   }
   return 0;
}

